import { Api, Get, Post, Query } from '@midwayjs/hooks';
import { Collection, Document, MongoClient, MongoClientOptions } from 'mongodb';
import { getAppCache, getQuery, getDbConnectKey, getRandomString } from '../util/util';
import { omit, get, pick } from 'lodash';
import { ObjectId } from 'mongodb';
export interface collectionOptions {
  key: string;
  db: string;
  collection: string;
}
const getCollection = (options: collectionOptions) => {
  const conn: MongoClient = getAppCache(options.key, getDbConnectKey());
  if (!conn) return;
  const db = conn.db(options.db);
  if (!db) return false;
  const collection = db.collection(options.collection);
  return collection;
};
export const query = Api(Get(), Query<any>(), async () => {
  const query = getQuery();
  const options = pick(query, ['key', 'db', 'collection']);
  const collection = getCollection(options);
  if (!collection) return;
  const filter = omit(query, ['key', 'db', 'collection', 'skip', 'limit']);
  const skip = get(query, 'skip', 0);
  const limit = get(query, 'limit', 0);
  const data = await collection.find(filter).sort({ 'meta.createdAt': -1 }).skip(parseInt(skip)).limit(parseInt(limit)).allowDiskUse().toArray();
  const total = await collection.countDocuments(filter);
  return { data, total };
});
export const create = Api(Post(), async (data: object, options: collectionOptions) => {
  const collection = getCollection(options);
  if (!collection) return;
  const result = await collection.insertOne(data);
  return result;
});
export const updateOne = Api(Post(), async (data: Document, options: collectionOptions) => {
  const collection = getCollection(options);
  if (!collection) return;
  const _id= new ObjectId(data._id);
  delete data._id
  const result = await collection.updateOne({ _id }, { $set: data });
  return result;
});
export const deleteOne = Api(Post(), async (data: Document, options: collectionOptions) => {
  const collection = getCollection(options);
  if (!collection) return;
  const _id= new ObjectId(data._id);
  const result = await collection.deleteOne({ _id });
  return result;
});
