import { Api, Get, Post, Query } from '@midwayjs/hooks';
import { MongoClient, MongoClientOptions } from 'mongodb';
import { dbConnectInfo } from '../interface/interface';
import { setAppCache, getAppCache, getQuery, getDbConnectKey, getRandomString } from '../util/util';
/**
 * mongodb连接
 */
export const DBConnect = Api(Post(), async (info: dbConnectInfo) => {
  let url = `mongodb://`;
  const options: MongoClientOptions = { monitorCommands: true };
  if (info.authDb) options.authSource = info.authDb;
  if (info.username && info.password) {
    // 使用url作为连接缓存的key
    url = `${url}${info.username}:${info.password}@`;
    // options.auth = { username: info.username, password: info.password }; // 另一种连接方式
  }
  url = `${url}${info.ip}:${info.port}`;
  try {
    let conn: MongoClient = getAppCache(url) as MongoClient;
    if (!conn) {
      const connection = await MongoClient.connect(url, options);
      const key = getRandomString();
      setAppCache(key, connection, getDbConnectKey());
      conn = connection;
      return { url, key };
    }
  } catch (error) {
    console.error(error);
    return '数据库连接失败';
  }
});

/**
 * 数据库查询
 */
export const DataBaseList = Api(
  Get(),
  Query<{
    key: string;
  }>(),
  async () => {
    const query = getQuery();
    const conn: MongoClient = getAppCache(query.key, getDbConnectKey()) as MongoClient;
    if (!conn) return;
    const db = conn.db('admin');
    const dataBaseList = await db.command({ listDatabases: 1 });
    return dataBaseList;
  }
);

export const getCollectionList = Api(Get(), Query<{ key: string; dbName: string }>(), async () => {
  const query = getQuery();
  const conn: MongoClient = getAppCache(query.key, getDbConnectKey()) as MongoClient;
  const db = conn.db(query.dbName);
  const res = await db.collections();
  const data = [];
  for (const i of res) {
    const obj = { name: i.collectionName, total: 0 };
    const total = await i.countDocuments();
    obj.total = total;
    data.push(obj);
  }
  return data;
});
