import { useContext, useConfig } from '@midwayjs/hooks';
import { Context } from '@midwayjs/koa';
import { head, set, get } from 'lodash';
import { randomBytes } from 'crypto';
/**数据库连接缓存key */
export const connectCacheKey = 'connectCacheKey';
/**操作枚举 */
enum operaEum {
  SET,
  GET,
}
/**获取路径数组 */
const getPathArray = (path) => {
  const arr = path.split('.');
  return arr;
};
/**处理有路径的存/储操作
 * @property object ctx/app
 * @property key 最后的属性值
 * @property path 到达key的路径
 * @property target 值
 * @property opera 操作
 */
const deal = (object: any, key: string, path: string, opera: operaEum, target?: any) => {
  const arr = getPathArray(path);
  const objectKey: string = head(arr);
  let obj: object = object.getAttr(objectKey);
  arr.shift();
  arr.push(key);
  const pathKey = arr.join('.');
  if (opera === operaEum.SET) {
    // 存
    if (obj) {
      obj = set(obj, pathKey, target);
    } else {
      obj = {};
      obj = set(obj, pathKey, target);
    }
    object.setAttr(objectKey, obj);
  } else if (opera === operaEum.GET) {
    // 取
    return get(obj, pathKey);
  }
};

export const setAppCache = (key: string, target: any, path?: string) => {
  const ctx = useContext<Context>();
  const app = ctx.getApp();
  if (path) {
    deal(app, key, path, operaEum.SET, target);
  } else {
    app.setAttr(key, target);
  }
  return 'ok';
};

export const getAppCache = (key: string, path?: string) => {
  const ctx = useContext<Context>();
  const app = ctx.getApp();
  if (path) {
    return deal(app, key, path, operaEum.GET);
  } else return app.getAttr(key);
};

export const setCtxCache = (key: string, target: any, path?: string) => {
  const ctx = useContext<Context>();
  if (path) {
    deal(ctx, key, path, operaEum.SET, target);
  } else {
    ctx.setAttr(key, target);
  }
  return 'ok';
};
export const getCtxCache = (key: string, path?: string) => {
  const ctx = useContext<Context>();
  if (path) {
    return deal(ctx, key, path, operaEum.GET);
  } else return ctx.getAttr(key);
};

export const getConfig = (key: string) => {
  return useConfig(key);
};
export const getDbConnectKey = () => {
  return useConfig('connectCacheKey');
};

export const getQuery = () => {
  const ctx = useContext();
  return ctx.query;
};

export const getRandomString = () => {
  return randomBytes(16).toString('hex').slice(0, 16);
};
